// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

//     Fichier contenant les routines de gestion SPI
//
// Ce fichier contient les fonctions specifiques pour le circuit PLL 145170
// ainsi que les fonctions d'acces a l'afficheur LCD
//=======================================
// History:
// 24/10/04    V1.0
//       Version originale
// 12/04/06    V1.1
//       adaptation pour ti_pfm
// 03/11/06    V1.2
//       regroupement de l'init LCD dans une fonction
//       mise en commentaires des traces de debug
// 22/11/06    V1.3
//       Optimisation du timing d'acces PLL et LCD
//       Modification lecture status LCD 
//=======================================

#include "TIPLL_spi.h"
#include "TIPLL_FreqMgt.h"
#include "TIPLL_USART2.h"
#include "TIPLL_IOMgt.h"

/* include system */
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>   

/* fonctions SPI master */
static unsigned char SPI_TmpConvertString[20];  /* conversion en ascii */

void SPI_MasterInit(void)
{
 
   /* Set MOSI and SCK output, all others input */
   /* port B utilise pour le ATmega8 */
   /* MOSI correspond a PB3 configure en sortie pour le AT mega8 */
   /* SCK  correspond a PB5 configure en sortie pour le ATmega8 */
   DDRB |= (1<<DDB3)|(1<<DDB5);
   /* mode SPI 0 pour MC145170 horloge active niveau haut et chgt des valeurs sur front descendant*/
   /* (le 145170 echantillonne la data sur un front montant de l'horloge) donc CPOL=0 et CPHA=1 */
   SPCR &= ~(1<<CPOL);
   /* Enable SPI, Master, set clock rate fck/128 */
   SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0);

 }

void SPI_MasterTransmitData(char cData)
{
// debug USART_PrintfStr(PSTR("SPI_MasterTransmitData:"));
// debug USART_PrintfUi_IT(((unsigned int )cData) & 0x00FF,1);
// debug USART_PrintfStr(PSTR("\r\n"));
   /* Wait for previous  transmission complete */
   while(!(SPSR & (1<<SPIF)));
 
  
   
   /* Start transmission */
   SPDR = cData;
   /* Wait for transmission complete */
   while(!(SPSR & (1<<SPIF)));
     
}




/* fonctions applicatives pour la carte Test Integration PLL */
/* ********************************************************* */

/* Fonctions specifiques pour le circuit MC145170 */

void SPI_PLL_select(void)
{
    /* selection du circuit PLL par mise a 0 de ENB */
	/* L'entree ENB correspond a la sortie PORTB.2 pour cette carte */

	PORTB &= ~(1<<2);
	/* 22/11/06 ajout attente prise en compte PLL select */ 
	SPI_PLL_delay(PLL_WAIT_ENB_SELECT);
}

void SPI_PLL_unselect(void)
{
   /* deselection du circuit PLL par mise a 1 de ENB */
   /* L'entree ENB correspond a la sortie PORTB.2 pour cette carte */

   PORTB |= (1<<2);
   /* 22/11/06 ajout attente prise en compte PLL unselect */ 
   SPI_PLL_delay(PLL_WAIT_ENB);
}

void SPI_PLL_delay(unsigned int Tempo)
{
   /* Cette fonction realise un temps d'attente afin de maintenir une sortie suffisament longtemps */
   /* pour permettre une prise en compte par le circuit */
   /* cf datasheet 145170 */
   /* temps min Din 55 ns sur un front montant de CLK */
   /* temps inactivite de ENB entre 2 ecriture 400 ns */
   
   while (Tempo !=0)
   {
         Tempo--;
   }		 
   
}


char SPI_setFreq(int Freq10KHZ)
{
  char RetCode=0;
  int DiviseurN=0;
   /* debug */
   // debug USART_PrintfStr(PSTR("SPI_setFreq :"));
   // debug USART_PrintfUi_IT(Freq10KHZ,0);
   // debug USART_PrintfStr(PSTR("\r\n"));
  SPI_LCD_DisplayCurFreq(Freq10KHZ);
 
  /* programmation PLL depuis le parametre Freq10KHZ */
  /* cette fonction retourne 0 si la frequence est OK et -1 en cas d'erreur */
  /* La frequence n'est valide unuquement entre 87.50 MHz et 108.00 MHz */
  /* toute valeur hors borne retourne une erreur */
 
  /* cette fonction ne programme que le registre N du MC145170, les autres registres sont supposes deja initialies*/
  if ((Freq10KHZ >= FREQ_FREQ_MIN) && (Freq10KHZ <= FREQ_FREQ_MAX))
  {
      /* calcul du diviseur N depuis Freq10KHZ ,PLL_FREQ_REF et PLL_REG_R*/ 
	  /* Attention l'ordre des operateurs est important pour eviter des debordements ou perte de precision*/
      /* le pas doit etre en 10Khz pour etre homogene donc *10 */
	  /* si *10 avant --> debordement 16b */
	  /* si /10 du pas perte precision pour pas de 25KHz */
	  DiviseurN = (Freq10KHZ/(PLL_PAS_KHZ/10));
	  
      /* select PLL */
	  SPI_PLL_select();
      /* ecriture partie haute */
      SPI_MasterTransmitData((char)(DiviseurN >>8));
	  SPI_PLL_delay(PLL_WAIT_ENB_SELECT);
      /* ecriture partie basse */
      SPI_MasterTransmitData((char)(DiviseurN & 0x00FF));

  
      /* deselect PLL */
	  SPI_PLL_unselect();
  }
  else
  {
      RetCode=-1;
  }
  return RetCode;
}

/* fonction de reset du MC145170 */
/* la sequence de reset n'est pas directement compatible avec le SPI (nbre de clock != 8) */
/* donc cette sequence est generee manuellement*/
void SPI_PLL_reset(void)
{
   char Cnt_clk;   /* compteur horloge pour trame CLK  */
// debug   USART_PrintfStr(PSTR("SPI_PLL_reset\r\n"));
    
   /* desactivation SPI */
   SPCR &= ~(1<<SPE);
   
   /* positionnement etat haut ENB */
   /* deselect PLL */
   SPI_PLL_unselect();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* positionnement etat bas de Din*/
   PORTB &= ~(1<<3);
   /* positionnement etat bas CLK */
   PORTB &= ~(1<<5);

   /* Test sur 6 pulse au lieu de 8 */
   /* trame  pulse CLK */
   for (Cnt_clk=6; Cnt_clk !=0 ; Cnt_clk--)
   {
       /* etat haut CLK */
	   PORTB |= (1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat bas CLK */
       PORTB &= ~(1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   }
   /* positionnement etat bas ENB */
   /* select PLL */
   SPI_PLL_select();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   
   /* trame 3 pulse CLK */
   for (Cnt_clk=3; Cnt_clk !=0 ; Cnt_clk--)
   {
       /* etat haut CLK */
	   PORTB |= (1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat bas CLK */
       PORTB &= ~(1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   }
   /* positionnement etat haut Din */
   PORTB |= (1<<3);
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   
   /* 1 pulse CLK */
   /* etat haut CLK */
   PORTB |= (1<<5);
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* etat bas CLK */
   PORTB &= ~(1<<5);
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/

   /* positionnement etat bas Din */
   PORTB &= ~(1<<3);
   /* 1 pulse CLK */
   /* etat haut CLK */
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   PORTB |= (1<<5);
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* etat bas CLK */
   PORTB &= ~(1<<5);
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/

   /* positionnement etat haut ENB (deselection circuit fin RESET) */
   /* deselect PLL */
   SPI_PLL_unselect();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
     
   /* reactivation SPI en master car reset avec le disable SPI */
   SPCR |= (1<<SPE)| (1<<MSTR);
}

/* cette fonction initialise le 145170 avec des valeurs par defaut pour les registe C et R */
void SPI_PLL_init(int RegistreN)
{
// debug   USART_PrintfStr(PSTR("SPI_PLL_init\r\n"));

   /* configuration PORTB.2 en sortie  pour selection ENB */
   DDRB |= (1<<2);
   
   /* reset circuit */
   SPI_PLL_reset();
    
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
 
   /* init registre N */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   SPI_MasterTransmitData((char)(RegistreN >>8));
   SPI_PLL_delay(PLL_WAIT_ENB_SELECT);
   SPI_MasterTransmitData((char)(RegistreN & 0x00FF));
   /* deselect PLL */
   SPI_PLL_unselect();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/

   /* init registre R */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   /* ecriture 1er octet pour fonction bit grabber (detection registre R sur acces 24 bits)*/
   SPI_MasterTransmitData(0);
   SPI_MasterTransmitData((char)(PLL_REG_R >>8));
   SPI_MasterTransmitData((char)(PLL_REG_R & 0x00FF));
   
   /* deselect PLL */
   SPI_PLL_unselect();
   
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/ 

/* debug*/
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
 
   /* init registre C */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   SPI_MasterTransmitData(PLL_REG_C);
   /* deselect PLL */
   SPI_PLL_unselect();
  
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
}

/* FONCTION SPECIFIQUE INTEGRATION */
/* cette fonction ecrit le registre C du 145170 */
void SPI_PLL_C(char RegistreC)
{
   /* init registre C */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   SPI_MasterTransmitData(RegistreC);
   /* deselect PLL */
   SPI_PLL_unselect();
}

/* FONCTION SPECIFIQUE INTEGRATION */
/* cette fonction ecrit le registre R du 145170 */
void SPI_PLL_R(int RegistreR)
{
   /* init registre R */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   /* ecriture 1er octet pour fonction bit grabber (detection registre R sur acces 24 bits)*/
   SPI_MasterTransmitData(0);
   SPI_MasterTransmitData((char)(RegistreR >>8));
   SPI_MasterTransmitData((char)(RegistreR & 0x00FF));
   /* deselect PLL */
   SPI_PLL_unselect();
}

/* FONCTION SPECIFIQUE INTEGRATION */
/* cette fonction ecrit le registre R du 145170 */
void SPI_PLL_R_debug_24b(int RegistreR)
{
   char Cnt_clk;   /* compteur horloge pour trame CLK  */
   unsigned int Msk;
   
   Msk= 0x8000;
   /* init registre R */
   /* --------------- */
   /* desactivation SPI */
   SPCR &= ~(1<<SPE);
   /* select PLL */
   SPI_PLL_select();
   /* ecriture 1er octet pour fonction bit grabber (detection registre R sur acces 24 bits)*/
   
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* positionnement etat bas de Din*/
   PORTB &= ~(1<<3);
   /* positionnement etat bas CLK */
   PORTB &= ~(1<<5);

   /* trame 8 pulse CLK */
   for (Cnt_clk=8; Cnt_clk !=0 ; Cnt_clk--)
   {
       /* etat haut CLK */
	   PORTB |= (1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat bas CLK */
       PORTB &= ~(1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   }
   /* trame 16 pulse CLK */
   for (Cnt_clk=16; Cnt_clk !=0 ; Cnt_clk--)
   {
       /* positionnement Din */
	   if ((RegistreR & Msk) !=0)
	   {
         PORTB |= (1<<3);
	   }
	   else
	   {
          PORTB &= ~(1<<3);
	   }
	   Msk=Msk>>1;
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat haut CLK */
	   PORTB |= (1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat bas CLK */
       PORTB &= ~(1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   }
   /* deselect PLL */
   SPI_PLL_unselect();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* reactivation SPI en master car reset avec le disable SPI */
   SPCR |= (1<<SPE)| (1<<MSTR);
}

/* FONCTION SPECIFIQUE INTEGRATION */
/* cette fonction ecrit le registre R du 145170 */
void SPI_PLL_R_debug(int RegistreR)
{
   char Cnt_clk;   /* compteur horloge pour trame CLK  */
   unsigned int Msk;
   
   Msk= 0x4000;
   /* init registre R */
   /* --------------- */
   /* desactivation SPI */
   SPCR &= ~(1<<SPE);
   /* select PLL */
   SPI_PLL_select();

   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* positionnement etat bas de Din*/
   PORTB &= ~(1<<3);
   /* positionnement etat bas CLK */
   PORTB &= ~(1<<5);


   /* trame 15 pulse CLK */
   for (Cnt_clk=15; Cnt_clk !=0 ; Cnt_clk--)
   {
       /* positionnement Din */
	   if ((RegistreR & Msk) !=0)
	   {
         PORTB |= (1<<3);
	   }
	   else
	   {
          PORTB &= ~(1<<3);
	   }
	   Msk=Msk>>1;
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat haut CLK */
	   PORTB |= (1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
       /* etat bas CLK */
       PORTB &= ~(1<<5);
       SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   }
   /* deselect PLL */
   SPI_PLL_unselect();
   SPI_PLL_delay(PLL_WAIT_ENB);  /* tempo ENB*/
   /* reactivation SPI en master car reset avec le disable SPI */
   SPCR |= (1<<SPE)| (1<<MSTR);
}



/* FONCTION SPECIFIQUE INTEGRATION */
/* cette fonction ecrit le registre N du 145170 */
void SPI_PLL_N(int RegistreN)
{
   /* init registre N */
   /* --------------- */
   /* select PLL */
   SPI_PLL_select();
   SPI_MasterTransmitData((char)(RegistreN >>8));
   SPI_MasterTransmitData((char)(RegistreN & 0x00FF));
   /* deselect PLL */
   SPI_PLL_unselect();
}




/* ******************************************** */
/* ** Fonctions driver LCD                   ** */
/* ******************************************** */


/* Init LCD */
void SPI_LCD_Init(void)
{
   SPI_LCD_SendCtrl(0x01);  /* reset */
   SPI_LCD_SendCtrl(0x0E);  /* display ON,Curseur ON, blink OFF*/
   SPI_LCD_SendCtrl(0x38);  /* DL =1 N=1 M=1 G=0 b0=0 */
   SPI_LCD_SendCtrl(0x06);  /* move cursor =1 autoshift=0*/
  
   SPI_LCD_Clear();

}


void SPI_LCD_SendData(char data)
{
   
   /* attente LCD ready */
   while (SPI_LCD_Read()!=0);

   PORTC &= ~(1<<IO_LCD_E);
   /* positionnement RW =0 */
   PORTC &= ~(1<<IO_LCD_RW);
      
   /* envoi data sur SPI */
   SPI_MasterTransmitData(data);
   /* attente fin de transfert integree dans la fct Mastertransmit*/
   
   /* positionnement RS =1 */
   PORTC |= (1<<IO_LCD_RS);
   
   /* generation creneau sur E */
   PORTC |= (1<<IO_LCD_E);
   SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo ENB*/
   PORTC &= ~(1<<IO_LCD_E);
   SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo ENB*/

   /* reset RS(0) et RW(1) */
   PORTC &= ~(1<<IO_LCD_RS);
   PORTC |= (1<<IO_LCD_RW);

};
void SPI_LCD_SendCtrl(char data)
{
  /* attente LCD ready */
   while (SPI_LCD_Read()!=0);
   PORTC &= ~(1<<IO_LCD_E);
   
   /* positionnement RW =0 */
   PORTC &= ~(1<<IO_LCD_RW);
   /* envoi data sur SPI */
   SPI_MasterTransmitData(data);
   /* attente fin de transfert integree dans la fct Mastertransmit*/
   
   /* positionnement RS =0 */
   PORTC &= ~(1<<IO_LCD_RS);
   
   /* generation creneau sur E */
   PORTC |= (1<<IO_LCD_E);
   SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo ENB*/
   PORTC &= ~(1<<IO_LCD_E);
   SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo ENB*/

   /* reset RW(1) */
   PORTC |= (1<<IO_LCD_RW);

};
char SPI_LCD_Read(void)
{
    
    char TmpReturn;   /* variable temporaire */
    
    PORTC &= ~(1<<IO_LCD_E);

    /* RS=0 RW=1, top E */
    PORTC &= ~(1<<IO_LCD_RS);
    /* positionnement RW =1 */
    PORTC |= (1<<IO_LCD_RW);
    PORTC |= (1<<IO_LCD_E);
    SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo */
    /* lecture status */
	/* modif PFM !! PORT D pour LCD_READY */
	/* GL 13/10/06 correction POTRD ->PORTB */
    TmpReturn = (PINB & (1<<IO_LCD_READY));
    PORTC &= ~(1<<IO_LCD_E);
    SPI_PLL_delay(PLL_WAIT_LCD);  /* tempo */

    return (TmpReturn);
	//return((PINB & (1<<IO_LCD_READY)));
};

void SPI_LCD_SendLine(char * PtChar)
{
    while (*PtChar !=0)
	{
       SPI_LCD_SendData(*PtChar);
       PtChar++;
	}
	

};
void SPI_LCD_Home(void)
{
    SPI_LCD_SendCtrl(0x02);  /* reset home cursor */
};
void SPI_LCD_Clear(void)
{
    SPI_LCD_SendCtrl(0x01);  /* reset LCD */
};
void SPI_LCD_ClearLine1(void)
{
    unsigned char Index;
	
    SPI_LCD_Home();
	for (Index=0; Index<40;Index++)
	{
	    SPI_LCD_SendData(' ');
	}
	SPI_LCD_Home();

};

void SPI_LCD_DisplayCurFreq(int Freq10KHZ)
{
   SPI_LCD_ClearLine1();
   /* alignement affichage si >=100 MHz */
   if (Freq10KHZ>=10000)
   {
      SPI_LCD_SendLine("Freq:");
   }
   else
   {
       SPI_LCD_SendLine("Freq: ");
   }
   
   SPI_LCD_PrintfUi(Freq10KHZ,0);
   SPI_LCD_SendLine("0 KHz");
}


void SPI_LCD_PrintfUi(unsigned int  Data, unsigned char Format )
{
    /* printf d'un unsigend int sur afficheur LCD selon format 0: 10, 1:hexa */

	
    switch (Format)
    {
      case 1:
       {
       	  SPI_LCD_SendLine("0x");
       	  utoa(Data,SPI_TmpConvertString,16);
       }
       break;
       case 0:
       default:
       {
       	  utoa(Data,SPI_TmpConvertString,10);
       }
    }
    SPI_LCD_SendLine(SPI_TmpConvertString);
}


